#include "OgreOcclusionCullingOctreeSceneManager.h"
#include "OgreMath.h"
#include "OgreIteratorWrappers.h"
#include "OgreRenderSystem.h"
#include "OgreCamera.h"

#include <windows.h>

namespace Ogre {
	//-----------------------------------------------------------------------
	OcclusionCullingOctreeSceneManager::OcclusionCullingOctreeSceneManager()
	{
		mOcclusionCullingOctreeSceneTraverser = 
			new OcclusionCullingOctreeSceneTraverser(this, mDestRenderSystem);

		//mDisplayNodes = true;
		//mShowBoundingBoxes = true;
		//mShowBoxes = true;
		//mMaxDepth = 20;
	}
	//-----------------------------------------------------------------------
	OcclusionCullingOctreeSceneManager::~OcclusionCullingOctreeSceneManager()
	{
		delete mOcclusionCullingOctreeSceneTraverser;
	}
	//-----------------------------------------------------------------------
	void OcclusionCullingOctreeSceneManager::_renderVisibleObjects(void)
	{
		//-- render background
		clearSpecialCaseRenderQueues();
		addSpecialCaseRenderQueue(RENDER_QUEUE_BACKGROUND);
		addSpecialCaseRenderQueue(RENDER_QUEUE_SKIES_EARLY);
	
		setSpecialCaseRenderQueueMode(SceneManager::SCRQM_INCLUDE);
		SceneManager::_renderVisibleObjects( );
		_deleteRenderedQueueGroups();

		//-- render visible objects (i.e., all but overlay)
		clearSpecialCaseRenderQueues();
		addSpecialCaseRenderQueue(RENDER_QUEUE_OVERLAY);
		setSpecialCaseRenderQueueMode(SceneManager::SCRQM_EXCLUDE);

		mOcclusionCullingOctreeSceneTraverser->setSceneRoot(mOctree);
		mOcclusionCullingOctreeSceneTraverser->renderScene(mCameraInProgress);

		_deleteRenderedQueueGroups();

		//-- render rest, e.g., overlay
		clearSpecialCaseRenderQueues();
		SceneManager::_renderVisibleObjects();
		//_deleteRenderedQueueGroups();
	}
	//-----------------------------------------------------------------------
	void OcclusionCullingOctreeSceneManager::_findVisibleObjects(Camera* cam, bool onlyShadowCasters)
	{
		// must be empty because objects are found and rendered in an interleaved fashion
		// in _renderVisibibleObjects 
	}
	//-----------------------------------------------------------------------
	void OcclusionCullingOctreeSceneManager::_updateSceneGraph(Camera* cam)
	{
		mOcclusionCullingOctreeSceneTraverser->setNumSceneNodes((int)mSceneNodes.size());
		mOcclusionCullingOctreeSceneTraverser->setRenderSystem(mDestRenderSystem);
        
		mOcclusionCullingOctreeSceneTraverser->setNumOctreeNodes(mNumOctreeNodes);

		OctreeSceneManager::_updateSceneGraph(cam);
	}
	//-----------------------------------------------------------------------
	bool OcclusionCullingOctreeSceneManager::setOption( const String & key, const void * val )
	{
		return mOcclusionCullingOctreeSceneTraverser->setOption( key, val) || OctreeSceneManager::setOption( key, val );
	}
	//-----------------------------------------------------------------------
	bool OcclusionCullingOctreeSceneManager::getOption( const String & key, void *val )
	{
		return mOcclusionCullingOctreeSceneTraverser->getOption( key, val) || OctreeSceneManager::getOption( key, val ) ;
	}
	//-----------------------------------------------------------------------
	bool OcclusionCullingOctreeSceneManager::getOptionValues( const String & key, StringVector  &refValueList )
	{
		return OctreeSceneManager::getOptionValues( key, refValueList );
	}
	//-----------------------------------------------------------------------
	bool OcclusionCullingOctreeSceneManager::getOptionKeys( StringVector & refKeys )
	{
		return mOcclusionCullingOctreeSceneTraverser->getOptionKeys ( refKeys ) || OctreeSceneManager::getOptionKeys( refKeys );
	}
}	
